@@ -34,18 +34,15 @@
 
 #include <Python.h>
 #include <assert.h>
+// TODO: Switch to newer numpy api
+#define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
 #include <numpy/arrayobject.h>
 #include <spglib.h>
 #include <stdio.h>
 
-#if (PY_MAJOR_VERSION < 3) && (PY_MINOR_VERSION < 6)
-#define PYUNICODE_FROMSTRING PyString_FromString
-#else
-#define PYUNICODE_FROMSTRING PyUnicode_FromString
-#endif
-
 static PyObject *py_get_version(PyObject *self, PyObject *args);
 static PyObject *py_get_dataset(PyObject *self, PyObject *args);
+static PyObject *py_get_layerdataset(PyObject *self, PyObject *args);
 static PyObject *py_get_magnetic_dataset(PyObject *self, PyObject *args);
 static PyObject *py_get_spacegroup_type(PyObject *self, PyObject *args);
 static PyObject *py_get_spacegroup_type_from_symmetry(PyObject *self,
@@ -83,15 +80,8 @@ struct module_state {
     PyObject *error;
 };
 
-#if PY_MAJOR_VERSION >= 3
-#define GETSTATE(m) ((struct module_state *)PyModule_GetState(m))
-#else
-#define GETSTATE(m) (&_state)
-static struct module_state _state;
-#endif
-
 static PyObject *error_out(PyObject *m) {
-    struct module_state *st = GETSTATE(m);
+    struct module_state *st = (struct module_state *)PyModule_GetState(m);
     PyErr_SetString(st->error, "something bad happened");
     return NULL;
 }
@@ -100,6 +90,8 @@ static PyMethodDef _spglib_methods[] = {
     {"error_out", (PyCFunction)error_out, METH_NOARGS, NULL},
     {"version", py_get_version, METH_VARARGS, "Spglib version"},
     {"dataset", py_get_dataset, METH_VARARGS, "Dataset for crystal symmetry"},
+    {"layerdataset", py_get_layerdataset, METH_VARARGS,
+     "Dataset for layer symmetry"},
     {"magnetic_dataset", py_get_magnetic_dataset, METH_VARARGS,
      "Magnetic dataset for crystal symmetry"},
     {"spacegroup_type", py_get_spacegroup_type, METH_VARARGS,
@@ -143,15 +135,13 @@ static PyMethodDef _spglib_methods[] = {
 
     {NULL, NULL, 0, NULL}};
 
-#if PY_MAJOR_VERSION >= 3
-
 static int _spglib_traverse(PyObject *m, visitproc visit, void *arg) {
-    Py_VISIT(GETSTATE(m)->error);
+    Py_VISIT(((struct module_state *)PyModule_GetState(m))->error);
     return 0;
 }
 
 static int _spglib_clear(PyObject *m) {
-    Py_CLEAR(GETSTATE(m)->error);
+    Py_CLEAR(((struct module_state *)PyModule_GetState(m))->error);
     return 0;
 }
 
@@ -165,35 +155,33 @@ static struct PyModuleDef moduledef = {PyModuleDef_HEAD_INIT,
                                        _spglib_clear,
                                        NULL};
 
-#define INITERROR return NULL
-
-PyObject *PyInit__spglib(void)
+// Define macro to make sure symbol is exported
+// TODO: Switch to C23 native
+#if defined(_WIN32) || defined(WIN32)
+// Note: actually gcc and intel seem to also supports this syntax.
+#ifdef _MSC_VER
+#define EXPORT __declspec(dllexport)
 #else
-#define INITERROR return
-
-void init_spglib(void)
+#define EXPORT __attribute__((dllexport))
 #endif
-{
+#else
+#define EXPORT __attribute__((visibility("default")))
+#endif
+
+EXPORT PyObject *PyInit__spglib(void) {
     struct module_state *st;
-#if PY_MAJOR_VERSION >= 3
     PyObject *module = PyModule_Create(&moduledef);
-#else
-    PyObject *module = Py_InitModule("_spglib", _spglib_methods);
-#endif
 
-    if (module == NULL) INITERROR;
+    if (module == NULL) return NULL;
 
-    st = GETSTATE(module);
+    st = (struct module_state *)PyModule_GetState(module);
 
     st->error = PyErr_NewException("_spglib.Error", NULL, NULL);
     if (st->error == NULL) {
         Py_DECREF(module);
-        INITERROR;
+        return NULL;
     }
-
-#if PY_MAJOR_VERSION >= 3
     return module;
-#endif
 }
 
 static PyObject *py_get_version(PyObject *self, PyObject *args) {
@@ -217,13 +205,8 @@ static PyObject *py_get_version(PyObject *self, PyObject *args) {
     return array;
 }
 
-static PyObject *py_get_dataset(PyObject *self, PyObject *args) {
-    int hall_number;
-    double symprec, angle_tolerance;
-    PyArrayObject *py_lattice;
-    PyArrayObject *py_positions;
-    PyArrayObject *py_atom_types;
-
+PyObject *build_python_list_from_dataset(SpglibDataset *dataset) {
+    int len_list = 21;
     PyObject *array, *vec, *mat, *rot, *trans, *wyckoffs, *equiv_atoms;
     PyObject *crystallographic_orbits;
     PyObject *site_symmetry_symbols, *primitive_lattice, *mapping_to_primitive;
@@ -233,30 +216,6 @@ static PyObject *py_get_dataset(PyObject *self, PyObject *args) {
     PyObject *std_rotation;
 
     int i, j, k, n;
-    double(*lat)[3];
-    double(*pos)[3];
-    int num_atom, len_list;
-    int *typat;
-    SpglibDataset *dataset;
-
-    if (!PyArg_ParseTuple(args, "OOOidd", &py_lattice, &py_positions,
-                          &py_atom_types, &hall_number, &symprec,
-                          &angle_tolerance)) {
-        return NULL;
-    }
-
-    lat = (double(*)[3])PyArray_DATA(py_lattice);
-    pos = (double(*)[3])PyArray_DATA(py_positions);
-    num_atom = PyArray_DIMS(py_positions)[0];
-    typat = (int *)PyArray_DATA(py_atom_types);
-
-    if ((dataset = spgat_get_dataset_with_hall_number(
-             lat, pos, typat, num_atom, hall_number, symprec,
-             angle_tolerance)) == NULL) {
-        Py_RETURN_NONE;
-    }
-
-    len_list = 21;
     array = PyList_New(len_list);
     n = 0;
 
@@ -266,11 +225,11 @@ static PyObject *py_get_dataset(PyObject *self, PyObject *args) {
     PyList_SetItem(array, n, PyLong_FromLong((long)dataset->hall_number));
     n++;
     PyList_SetItem(array, n,
-                   PYUNICODE_FROMSTRING(dataset->international_symbol));
+                   PyUnicode_FromString(dataset->international_symbol));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(dataset->hall_symbol));
+    PyList_SetItem(array, n, PyUnicode_FromString(dataset->hall_symbol));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(dataset->choice));
+    PyList_SetItem(array, n, PyUnicode_FromString(dataset->choice));
     n++;
 
     /* Transformation matrix */
@@ -335,7 +294,7 @@ static PyObject *py_get_dataset(PyObject *self, PyObject *args) {
         PyList_SetItem(wyckoffs, i,
                        PyLong_FromLong((long)dataset->wyckoffs[i]));
         PyList_SetItem(site_symmetry_symbols, i,
-                       PYUNICODE_FROMSTRING(dataset->site_symmetry_symbols[i]));
+                       PyUnicode_FromString(dataset->site_symmetry_symbols[i]));
         PyList_SetItem(equiv_atoms, i,
                        PyLong_FromLong((long)dataset->equivalent_atoms[i]));
         PyList_SetItem(
@@ -421,11 +380,100 @@ static PyObject *py_get_dataset(PyObject *self, PyObject *args) {
     /* PyList_SetItem(array, n, PyLong_FromLong((long)
      * dataset->pointgroup_number)); */
     /* n++; */
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(dataset->pointgroup_symbol));
+    PyList_SetItem(array, n, PyUnicode_FromString(dataset->pointgroup_symbol));
     n++;
 
     assert(n == len_list);
+    return array;
+}
 
+static PyObject *py_get_dataset(PyObject *self, PyObject *args) {
+    int hall_number;
+    double symprec, angle_tolerance;
+    PyArrayObject *py_lattice;
+    PyArrayObject *py_positions;
+    PyArrayObject *py_atom_types;
+    PyObject *array;
+    double(*lat)[3];
+    double(*pos)[3];
+    int num_atom;
+    int *typat;
+    SpglibDataset *dataset;
+
+    if (!PyArg_ParseTuple(args, "OOOidd", &py_lattice, &py_positions,
+                          &py_atom_types, &hall_number, &symprec,
+                          &angle_tolerance)) {
+        return NULL;
+    }
+
+    lat = (double(*)[3])PyArray_DATA(py_lattice);
+    pos = (double(*)[3])PyArray_DATA(py_positions);
+    num_atom = PyArray_DIMS(py_positions)[0];
+    typat = (int *)PyArray_DATA(py_atom_types);
+
+    if ((dataset = spgat_get_dataset_with_hall_number(
+             lat, pos, typat, num_atom, hall_number, symprec,
+             angle_tolerance)) == NULL) {
+        Py_RETURN_NONE;
+    }
+
+    array = build_python_list_from_dataset(dataset);
+    spg_free_dataset(dataset);
+
+    return array;
+}
+
+static PyObject *py_get_layerdataset(PyObject *self, PyObject *args) {
+    int aperiodic_dir;
+    double symprec;
+    PyArrayObject *py_lattice;
+    PyArrayObject *py_positions;
+    PyArrayObject *py_atom_types;
+
+    PyObject *array;
+
+    double(*lat)[3];
+    double(*pos)[3];
+    int num_atom, len_list;
+    int *typat;
+    SpglibDataset *dataset;
+
+    if (!PyArg_ParseTuple(args, "OOOid", &py_lattice, &py_positions,
+                          &py_atom_types, &aperiodic_dir, &symprec)) {
+        return NULL;
+    }
+
+    if (!PyArray_IS_C_CONTIGUOUS(py_lattice)) {
+        PyErr_SetString(PyExc_RuntimeError, "Lattice vector not C_CONTIGUOUS");
+        return NULL;
+    }
+    if (!PyArray_IS_C_CONTIGUOUS(py_positions)) {
+        PyErr_SetString(PyExc_RuntimeError,
+                        "Positions vector not C_CONTIGUOUS");
+        return NULL;
+    }
+    if (!PyArray_IS_C_CONTIGUOUS(py_atom_types)) {
+        PyErr_SetString(PyExc_RuntimeError,
+                        "Atom types vector not C_CONTIGUOUS");
+        return NULL;
+    }
+
+    lat = (double(*)[3])PyArray_DATA(py_lattice);
+    pos = (double(*)[3])PyArray_DATA(py_positions);
+    num_atom = PyArray_DIMS(py_positions)[0];
+    typat = (int *)PyArray_DATA(py_atom_types);
+
+    dataset = spg_get_layer_dataset(lat, pos, typat, num_atom, aperiodic_dir,
+                                    symprec);
+
+    if (!dataset) {
+        Py_RETURN_NONE;
+    }
+
+    // NOTE: This code is still experimental. It parses the information
+    // from layer dataset exactly as it would from normal dataset.
+    // It should be checked that all of this information makes sense.
+    array = build_python_list_from_dataset(dataset);
     spg_free_dataset(dataset);
 
     return array;
@@ -692,25 +740,25 @@ static PyObject *py_get_spacegroup_type(PyObject *self, PyObject *args) {
     PyList_SetItem(array, n, PyLong_FromLong((long)spg_type.number));
     n++;
     PyList_SetItem(array, n,
-                   PYUNICODE_FROMSTRING(spg_type.international_short));
+                   PyUnicode_FromString(spg_type.international_short));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(spg_type.international_full));
+    PyList_SetItem(array, n, PyUnicode_FromString(spg_type.international_full));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(spg_type.international));
+    PyList_SetItem(array, n, PyUnicode_FromString(spg_type.international));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(spg_type.schoenflies));
+    PyList_SetItem(array, n, PyUnicode_FromString(spg_type.schoenflies));
     n++;
     PyList_SetItem(array, n, PyLong_FromLong((long)spg_type.hall_number));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(spg_type.hall_symbol));
+    PyList_SetItem(array, n, PyUnicode_FromString(spg_type.hall_symbol));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(spg_type.choice));
+    PyList_SetItem(array, n, PyUnicode_FromString(spg_type.choice));
     n++;
     PyList_SetItem(array, n,
-                   PYUNICODE_FROMSTRING(spg_type.pointgroup_international));
+                   PyUnicode_FromString(spg_type.pointgroup_international));
     n++;
     PyList_SetItem(array, n,
-                   PYUNICODE_FROMSTRING(spg_type.pointgroup_schoenflies));
+                   PyUnicode_FromString(spg_type.pointgroup_schoenflies));
     n++;
     PyList_SetItem(
         array, n,
@@ -718,7 +766,7 @@ static PyObject *py_get_spacegroup_type(PyObject *self, PyObject *args) {
     n++;
     PyList_SetItem(
         array, n,
-        PYUNICODE_FROMSTRING(spg_type.arithmetic_crystal_class_symbol));
+        PyUnicode_FromString(spg_type.arithmetic_crystal_class_symbol));
     n++;
 
     return array;
@@ -759,25 +807,25 @@ static PyObject *py_get_spacegroup_type_from_symmetry(PyObject *self,
     PyList_SetItem(array, n, PyLong_FromLong((long)spg_type.number));
     n++;
     PyList_SetItem(array, n,
-                   PYUNICODE_FROMSTRING(spg_type.international_short));
+                   PyUnicode_FromString(spg_type.international_short));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(spg_type.international_full));
+    PyList_SetItem(array, n, PyUnicode_FromString(spg_type.international_full));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(spg_type.international));
+    PyList_SetItem(array, n, PyUnicode_FromString(spg_type.international));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(spg_type.schoenflies));
+    PyList_SetItem(array, n, PyUnicode_FromString(spg_type.schoenflies));
     n++;
     PyList_SetItem(array, n, PyLong_FromLong((long)spg_type.hall_number));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(spg_type.hall_symbol));
+    PyList_SetItem(array, n, PyUnicode_FromString(spg_type.hall_symbol));
     n++;
-    PyList_SetItem(array, n, PYUNICODE_FROMSTRING(spg_type.choice));
+    PyList_SetItem(array, n, PyUnicode_FromString(spg_type.choice));
     n++;
     PyList_SetItem(array, n,
-                   PYUNICODE_FROMSTRING(spg_type.pointgroup_international));
+                   PyUnicode_FromString(spg_type.pointgroup_international));
     n++;
     PyList_SetItem(array, n,
-                   PYUNICODE_FROMSTRING(spg_type.pointgroup_schoenflies));
+                   PyUnicode_FromString(spg_type.pointgroup_schoenflies));
     n++;
     PyList_SetItem(
         array, n,
@@ -785,7 +833,7 @@ static PyObject *py_get_spacegroup_type_from_symmetry(PyObject *self,
     n++;
     PyList_SetItem(
         array, n,
-        PYUNICODE_FROMSTRING(spg_type.arithmetic_crystal_class_symbol));
+        PyUnicode_FromString(spg_type.arithmetic_crystal_class_symbol));
     n++;
 
     return array;
@@ -810,8 +858,8 @@ static PyObject *py_get_magnetic_spacegroup_type(PyObject *self,
     n = 0;
     PyList_SetItem(array, n++, PyLong_FromLong((long)msg_type.uni_number));
     PyList_SetItem(array, n++, PyLong_FromLong((long)msg_type.litvin_number));
-    PyList_SetItem(array, n++, PYUNICODE_FROMSTRING(msg_type.bns_number));
-    PyList_SetItem(array, n++, PYUNICODE_FROMSTRING(msg_type.og_number));
+    PyList_SetItem(array, n++, PyUnicode_FromString(msg_type.bns_number));
+    PyList_SetItem(array, n++, PyUnicode_FromString(msg_type.og_number));
     PyList_SetItem(array, n++, PyLong_FromLong((long)msg_type.number));
     PyList_SetItem(array, n++, PyLong_FromLong((long)msg_type.type));
 
@@ -848,7 +896,7 @@ static PyObject *py_get_pointgroup(PyObject *self, PyObject *args) {
     }
 
     array = PyList_New(3);
-    PyList_SetItem(array, 0, PYUNICODE_FROMSTRING(symbol));
+    PyList_SetItem(array, 0, PyUnicode_FromString(symbol));
     PyList_SetItem(array, 1, PyLong_FromLong((long)ptg_num));
     PyList_SetItem(array, 2, mat);
 
@@ -1397,5 +1445,5 @@ static PyObject *py_get_error_message(PyObject *self, PyObject *args) {
 
     error = spg_get_error_code();
 
-    return PYUNICODE_FROMSTRING(spg_get_error_message(error));
+    return PyUnicode_FromString(spg_get_error_message(error));
 }
