@@ -35,8 +35,10 @@
 #ifndef __debug_H__
 #define __debug_H__
 
+#include <stdio.h>
+
+// Macros to either print or do nothing if in debug mode or not
 #ifdef SPGDEBUG
-#define NIGGLI_DEBUG
 #define debug_print(...) printf(__VA_ARGS__)
 #define debug_print_matrix_d3(a) dbg_print_matrix_d3(a)
 #define debug_print_matrix_i3(a) dbg_print_matrix_i3(a)
@@ -45,11 +47,12 @@
 #define debug_print_vectors_with_label(...) \
     dbg_print_vectors_with_label(__VA_ARGS__)
 
-void dbg_print_matrix_d3(double a[3][3]);
-void dbg_print_matrix_i3(int a[3][3]);
-void dbg_print_vector_d3(double a[3]);
-void dbg_print_vectors_d3(double a[][3], int size);
-void dbg_print_vectors_with_label(double a[][3], int b[], int size);
+// Just to make sure these are never used, these definitions are #ifdef guarded
+void dbg_print_matrix_d3(const double a[3][3]);
+void dbg_print_matrix_i3(const int a[3][3]);
+void dbg_print_vector_d3(const double a[3]);
+void dbg_print_vectors_d3(const double a[][3], int size);
+void dbg_print_vectors_with_label(const double a[][3], const int b[], int size);
 
 #else
 #define debug_print(...)
@@ -61,9 +64,6 @@ void dbg_print_vectors_with_label(double a[][3], int b[], int size);
 #endif
 
 #ifdef SPGWARNING
-#define NIGGLI_WARNING
-#include <stdio.h>
-/* #define warning_print(...) fprintf (stderr, __VA_ARGS__) */
 #define warning_print(...) fprintf(stderr, __VA_ARGS__)
 #else
 #define warning_print(...)
